/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package p4uclient;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;

/**
 * ファイル管理
 * @author tondol
 */
public class FileManager {
    private String filename = null;
    private ArrayList<FileItem> items = null;
    
    /**
     * アップロードリストを読み込む
     */
    public FileManager(String filename) {
        this.filename = filename;
        items = new ArrayList<FileItem>();
        FileReader     ifstream = null;
        BufferedReader reader   = null;
        try {
            ifstream = new FileReader(filename);
            reader   = new BufferedReader(ifstream);
            while (reader.ready()) {
                String line = reader.readLine();
                addItem(new FileItem(line));
            }
            MainApp.window.putMessage("o リスト読込");
        } catch(Exception e) {
            MainApp.window.putMessage("x リスト読込");
        } finally {
            try {
                if (reader   != null) reader.close();
                if (ifstream != null) ifstream.close();
                updateItems();
            } catch(Exception e) {
            }
        }
    }
    
    /**
     * アップロードリストを書き込む
     */
    public void close() throws Exception {
        FileWriter  ofstream = null;
        PrintWriter writer   = null;
        try {
            ofstream = new FileWriter(filename);
            writer   = new PrintWriter(ofstream);
            for (int i=0;i<items.size();i++) {
                FileItem item = items.get(i);
                writer.println(item.toString());
            }
            MainApp.window.putMessage("o リスト書込");
        } catch(Exception e) {
            MainApp.window.putMessage("x リスト書込");
        } finally {
            try {
                if (writer   != null) writer.close();
                if (ofstream != null) ofstream.close();
                clearItems();
            } catch(Exception e) {
            }
        }
    }
    
    /**
     * アイテムを追加
     */
    private void addItem(FileItem item) throws Exception {
        items.add(item);
        MainApp.client.addItem(item);
    }
    
    /**
     * アイテムを削除
     */
    private void deleteItem(FileItem item) throws Exception {
        items.remove(item);
        MainApp.client.delItem(item);
    }
    
    /**
     * ハッシュからファイルを取得
     */
    public FileItem getItem(String digest) {
        FileItem ret = null;
        for (int i=0;i<items.size();i++){
            if (digest.equals(items.get(i).hash)) {
                ret = items.get(i);
            }
        }
        return ret;
    }
    
    /**
     * アイテムをクリア
     */
    public void clearItems() throws Exception {
        while (!items.isEmpty()) {
            int index = items.size() - 1;
            deleteItem(items.get(index));
        }
    }
    
    /**
     * 公開アイテムを更新
     */
    public void updateItems() throws Exception {
        // ファイルリストを取得
        File directory = new File(MainApp.option.updir);
        File array[]   = directory.listFiles();
        // 元リストを複製
        ArrayList<FileItem> tmp = new ArrayList<FileItem>();
        for (int i=0;i<items.size();i++) {
            tmp.add(items.get(i));
        }
        // 追加
        for (int i=0;i<array.length;i++) {
            File item = array[i];
            if (!contains(tmp, item)) {
                addItem(new FileItem(item));
            }
        }
        // 削除
        for (int i=0;i<tmp.size();i++) {
            FileItem item = tmp.get(i);
            if (!contains(array, item)) {
                deleteItem(item);
            }
        }
    }
    
    /**
     * ファイル名だけで包括チェック
     */
    private boolean contains(ArrayList<FileItem> list, File file) {
        for (int i=0;i<list.size();i++) {
            String name  = list.get(i).name;
            String other = file.getName();
            if (name.equals(other)) return true;
        }
        return false;
    }
    private boolean contains(File[] list, FileItem item) {
        for (int i=0;i<list.length;i++) {
            String name  = list[i].getName();
            String other = item.name;
            if (name.equals(other)) return true;
        }
        return false;
    }
}
